RMS.LoadLibrary("rmgen");

// initialize map
log("Initializing map...");
InitMap();


// Set some variables
var numPlayers = getNumPlayers();
var mapSize = getMapSize();
var mapRadius = getMapSize()/2;
var mapCenterX = mapRadius;
var mapCenterZ = mapRadius;
var mapCenter = [mapCenterX, mapCenterZ];

// Helper functions

// Funtion to place civil centre and starting units at given location for a given player
function myPlaceStartEntities(startLocation, playerId)
// "startLocation" should be a list of length 2 with floats determining the 2d-point where the starting entities shall be placed.
// "startLocation[0]" should be an float determining the x coordinate of the start location.
// "startLocation[1]" should be an float determining the z coordinate of the start location. (Should by y in my book though)
// "playerId" should be the player number like seen in Atlas or the game settings ingame (Starting with 1 for the first player with default colour blue, 0 is Gaia)
{
	// Some checks. Enhance if needed...
	
	// Setting some variables
	// Setting default building angle, can be got from the global "BUILDING_ANGlE" in further versions
	var buildingOrientation = 3*Math.PI/4;
	var buildingOrientation = playerAngle[playerId-1] + PI;
	// Getting civilisation dependent default starting entities
	var civEntities = g_CivData[g_MapSettings.PlayerData[i].Civ].StartEntities;
	
	// Place civil centre on the start location
	placeObject(startLocation[0], startLocation[1], civEntities[0].Template, playerId, buildingOrientation);
	
	// Place starting units
	// Minimal distance to start location so that units don't stand in the civil centere
	var minDistToCC = 4;
	// Minimal distance between units so they don't overlap
	var minDist = 1;
	// Set angle width units should be spread over
	var angleSpread = PI/4;
	// Outer loop for the diffrent unit types to place beside the civil centre (so starting with 1)
	for (var unitTypeIndex = 1; unitTypeIndex < civEntities.length; ++unitTypeIndex)
	{
		// Reset default number of units of that type
		var unitCount = (civEntities[unitTypeIndex].Count !== undefined ? civEntities[unitTypeIndex].Count : 1);
		// Reset the angle added per unit of that type depending on the number of them, for multiple units of that type only (otherwise 0 devision)
		if (unitCount > 1) {var angleAdd = angleSpread/(unitCount-1)};
		// Inner loop for multiple units of the same type
		for (var unitIndex = 0; unitIndex < unitCount; unitIndex++)
		{
			// Resetting the angle the unit will be placed in comperison to the city centre
			if (unitCount > 1)
				var angleActual = buildingOrientation - angleSpread/2 + unitIndex*angleAdd;
			else
				var angleActual = buildingOrientation;
			// Place the unit
			placeObject(
				startLocation[0] + (minDistToCC + minDist*unitTypeIndex)*sin(angleActual),
				startLocation[1] + (minDistToCC + minDist*unitTypeIndex)*cos(angleActual),
				civEntities[unitTypeIndex].Template, playerId, angleActual);
		};
	};
};

// Setting up players start position
var playerStartLocation = new Array(numPlayers);
var playerAngle = new Array(numPlayers);
var startAngle = randFloat(0, 2*PI);
var angleDist = 2*PI/numPlayers;
var fortressById = ['demo', 'small', 'very large', 'normal', 'large', 'medium', 'giant', 'tiny']
for (var i=0; i < numPlayers; i++)
{
	playerAngle[i] = i*angleDist + startAngle;
	var x = mapCenterX + 0.6*mapRadius*sin(playerAngle[i]);
	var z = mapCenterZ + 0.6*mapRadius*cos(playerAngle[i]);
	playerStartLocation[i] = [x, z];
	
	myPlaceStartEntities(playerStartLocation[i], i+1)
	var civ = g_MapSettings.PlayerData[i].Civ;
	if (civ == 'rome')
		civ = 'palisades'
	var myWall = new wallTool(civ);
	myWall.setFortress(fortressById[i]);
	myWall.place(playerStartLocation[i][0], playerStartLocation[i][1], i+1, playerAngle[i]);
};

// Test the wallTool
var myWall = new wallTool("palisades");
myWall.setStyle("palisades");
myWall.fortressTypes["my fortress"] = new fortress("really my fortress"); // Here you see that 'fortress.name' name is not really usefull...
myWall.fortressTypes["my fortress"].wall = [];
var wallPart = [];
// var wallPart = ['entry', 'endLeft', 'wall', 'cornerIn', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'cornerIn', 'wall', 'cornerOut', 'wall', 'cornerIn', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'cornerOut', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'gate', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'gate', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'outpost', 'gate', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'gate', 'outpost', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'cornerOut', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'gate', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'gate', 'outpost', 'wall', 'tower', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'cornerOut', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'tower', 'gate', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'cornerOut', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'gate', 'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];

// var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'tower', 'gate', 'tower', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'cornerOut', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'tower', 'gate', 'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];

var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'wallFort', 'wall', 'tower', 'gate', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'endRight', 'wideEntry', 'endLeft', 'wall', 'tower', 'wall', 'cornerOut', 'wall', 'tower', 'wall', 'endRight', 'wideEntry', 'endLeft', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'gate', 'tower', 'wall', 'wallFort', 'wall', 'tower', 'wall', 'endRight'];

for (var iPart = 0; iPart < 4; iPart++)
{
	for (var iElement = 0; iElement < wallPart.length; iElement++)
		myWall.fortressTypes["my fortress"].wall.push(wallPart[iElement]);
};
myWall.setFortress("my fortress");
myWall.place(mapCenterX, mapCenterX, 1, 0*PI/2);
// myWall.setFortress("tiny");
// myWall.place(mapCenterX, mapCenterX, 1, 0*PI/2);


// Export map data
ExportMap();
